@@ -1,7 +1,7 @@
 /* Copyright 2001,2002,2003 Roger Dingledine, Matej Pfajfar.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson */
 /* See LICENSE for licensing information */
-/* $Id: crypto.h,v 1.72 2005/12/08 17:38:31 nickm Exp $ */
+/* $Id: crypto.h,v 1.74 2006/01/10 21:12:06 nickm Exp $ */
 
 /**
  * \file crypto.h
@@ -11,10 +11,12 @@
 
 #ifndef __CRYPTO_H
 #define __CRYPTO_H
-#define CRYPTO_H_ID "$Id: crypto.h,v 1.72 2005/12/08 17:38:31 nickm Exp $"
+#define CRYPTO_H_ID "$Id: crypto.h,v 1.74 2006/01/10 21:12:06 nickm Exp $"
 
 #include <stdio.h>
 
+#undef ENABLE_0119_PARANOIA
+
 /** Length of the output of our message digest. */
 #define DIGEST_LEN 20
 /** Length of our symmetric cipher's keys. */
@@ -61,7 +63,8 @@ crypto_pk_env_t *crypto_new_pk_env(void);
 void crypto_free_pk_env(crypto_pk_env_t *env);
 
 /* convenience function: wraps crypto_create_crypto_env, set_key, and init. */
-crypto_cipher_env_t *crypto_create_init_cipher(const char *key, int encrypt_mode);
+crypto_cipher_env_t *crypto_create_init_cipher(const char *key,
+                                               int encrypt_mode);
 
 crypto_cipher_env_t *crypto_new_cipher_env(void);
 void crypto_free_cipher_env(crypto_cipher_env_t *env);
@@ -69,10 +72,14 @@ void crypto_free_cipher_env(crypto_cipher_env_t *env);
 /* public key crypto */
 int crypto_pk_generate_key(crypto_pk_env_t *env);
 
-int crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env, const char *keyfile);
-int crypto_pk_write_public_key_to_string(crypto_pk_env_t *env, char **dest, size_t *len);
-int crypto_pk_read_public_key_from_string(crypto_pk_env_t *env, const char *src, size_t len);
-int crypto_pk_write_private_key_to_filename(crypto_pk_env_t *env, const char *fname);
+int crypto_pk_read_private_key_from_filename(crypto_pk_env_t *env,
+                                             const char *keyfile);
+int crypto_pk_write_public_key_to_string(crypto_pk_env_t *env,
+                                         char **dest, size_t *len);
+int crypto_pk_read_public_key_from_string(crypto_pk_env_t *env,
+                                          const char *src, size_t len);
+int crypto_pk_write_private_key_to_filename(crypto_pk_env_t *env,
+                                            const char *fname);
 int crypto_pk_DER64_encode_public_key(crypto_pk_env_t *env, char **dest);
 crypto_pk_env_t *crypto_pk_DER64_decode_public_key(const char *in);
 
