@@ -2,8 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: circuitlist.c,v 1.85 2005/12/10 09:36:25 arma Exp $ */
-const char circuitlist_c_id[] = "$Id: circuitlist.c,v 1.85 2005/12/10 09:36:25 arma Exp $";
+/* $Id: circuitlist.c,v 1.89 2006/01/05 21:23:03 nickm Exp $ */
+const char circuitlist_c_id[] =
+  "$Id: circuitlist.c,v 1.89 2006/01/05 21:23:03 nickm Exp $";
 
 /**
  * \file circuitlist.c
@@ -37,8 +38,8 @@ typedef struct orconn_circid_circuit_map_t {
   circuit_t *circuit;
 } orconn_circid_circuit_map_t;
 
-/** Helper for hash tables: compare the OR connection and circuit ID for a and b,
- * and return less than, equal to, or greater than zero appropriately.
+/** Helper for hash tables: compare the OR connection and circuit ID for a and
+ * b, and return less than, equal to, or greater than zero appropriately.
  */
 static INLINE int
 _orconn_circid_entries_eq(orconn_circid_circuit_map_t *a,
@@ -53,7 +54,8 @@ _orconn_circid_entry_hash(orconn_circid_circuit_map_t *a)
   return (((unsigned)a->circ_id)<<16) ^ (unsigned)(uintptr_t)(a->or_conn);
 }
 
-static HT_HEAD(orconn_circid_map, orconn_circid_circuit_map_t) orconn_circid_circuit_map = HT_INITIALIZER();
+static HT_HEAD(orconn_circid_map, orconn_circid_circuit_map_t)
+     orconn_circid_circuit_map = HT_INITIALIZER();
 HT_PROTOTYPE(orconn_circid_map, orconn_circid_circuit_map_t, node,
              _orconn_circid_entry_hash, _orconn_circid_entries_eq);
 HT_GENERATE(orconn_circid_map, orconn_circid_circuit_map_t, node,
@@ -242,7 +244,8 @@ circuit_new(uint16_t p_circ_id, connection_t *p_conn)
   if (p_conn) {
     circuit_set_circid_orconn(circ, p_circ_id, p_conn, P_CONN_CHANGED);
   }
-  /* circ->n_circ_id remains 0 because we haven't identified the next hop yet */
+  /* circ->n_circ_id remains 0 because we haven't identified the next hop
+   * yet */
 
   circ->package_window = CIRCWINDOW_START;
   circ->deliver_window = CIRCWINDOW_START;
@@ -280,6 +283,7 @@ circuit_free(circuit_t *circ)
   tor_free(circ->onionskin);
   circuit_free_cpath(circ->cpath);
   if (circ->rend_splice) {
+    tor_assert(circ->rend_splice->magic == CIRCUIT_MAGIC);
     circ->rend_splice->rend_splice = NULL;
   }
   /* Remove from map. */
@@ -326,8 +330,10 @@ circuit_free_all(void)
     circuit_free(global_circuitlist);
     global_circuitlist = next;
   }
-  smartlist_free(circuits_pending_or_conns);
-  circuits_pending_or_conns = NULL;
+  if (circuits_pending_or_conns) {
+    smartlist_free(circuits_pending_or_conns);
+    circuits_pending_or_conns = NULL;
+  }
   HT_CLEAR(orconn_circid_map, &orconn_circid_circuit_map);
 }
 
@@ -464,7 +470,7 @@ circuit_get_by_edge_conn(connection_t *conn)
  * been marked already.
  */
 void
-circuit_unlink_all_from_or_conn(connection_t *conn)
+circuit_unlink_all_from_or_conn(connection_t *conn, int reason)
 {
   circuit_t *circ;
   for (circ = global_circuitlist; circ; circ = circ->next) {
@@ -474,7 +480,7 @@ circuit_unlink_all_from_or_conn(connection_t *conn)
       if (circ->p_conn == conn)
         circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
       if (!circ->marked_for_close)
-        circuit_mark_for_close(circ);
+        circuit_mark_for_close(circ, reason);
     }
   }
 }
@@ -559,8 +565,9 @@ circuit_find_to_cannibalize(uint8_t purpose, extend_info_t *info,
   circuit_t *circ;
   circuit_t *best=NULL;
 
-  debug(LD_CIRC,"Hunting for a circ to cannibalize: purpose %d, uptime %d, capacity %d, internal %d",
-         purpose, need_uptime, need_capacity, internal);
+  debug(LD_CIRC,"Hunting for a circ to cannibalize: purpose %d, uptime %d, "
+        "capacity %d, internal %d",
+        purpose, need_uptime, need_capacity, internal);
 
   for (circ=global_circuitlist; circ; circ = circ->next) {
     if (CIRCUIT_IS_ORIGIN(circ) &&
@@ -600,7 +607,7 @@ circuit_mark_all_unused_circs(void)
     if (CIRCUIT_IS_ORIGIN(circ) &&
         !circ->marked_for_close &&
         !circ->timestamp_dirty)
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, END_CIRC_AT_ORIGIN);
   }
 }
 
@@ -641,7 +648,8 @@ circuit_expire_all_dirty_circs(void)
  *     rendezvous stream), then mark the other circuit to close as well.
  */
 void
-_circuit_mark_for_close(circuit_t *circ, int line, const char *file)
+_circuit_mark_for_close(circuit_t *circ, int reason, int line,
+                        const char *file)
 {
   connection_t *conn;
 
@@ -656,6 +664,22 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
         circ->marked_for_close_file, circ->marked_for_close);
     return;
   }
+  if (reason == END_CIRC_AT_ORIGIN) {
+    if (!CIRCUIT_IS_ORIGIN(circ)) {
+      warn(LD_BUG, "Specified 'at-origin' non-reason for ending circuit, "
+           "but circuit was not at origin. (called %s:%d, purpose=%d)",
+           file, line, circ->purpose);
+    }
+    reason = END_CIRC_REASON_NONE;
+  } else if (CIRCUIT_IS_ORIGIN(circ) && reason != END_CIRC_REASON_NONE) {
+    /* Don't warn about this; there are plenty of places where our code
+     * is origin-agnosic. */
+    reason = END_CIRC_REASON_NONE;
+  }
+  if (reason < _END_CIRC_REASON_MIN || reason > _END_CIRC_REASON_MAX) {
+    warn(LD_BUG, "Reason %d out of range at %s:%d", reason, file, line);
+    reason = END_CIRC_REASON_NONE;
+  }
 
   if (circ->state == CIRCUIT_STATE_ONIONSKIN_PENDING) {
     onion_pending_remove(circ);
@@ -682,7 +706,8 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
     tor_assert(circ->state == CIRCUIT_STATE_OPEN);
     tor_assert(circ->build_state->chosen_exit);
     /* treat this like getting a nack from it */
-    info(LD_REND,"Failed intro circ %s to %s (awaiting ack). Removing from descriptor.",
+    info(LD_REND, "Failed intro circ %s to %s (awaiting ack). "
+         "Removing from descriptor.",
          safe_str(circ->rend_query),
          safe_str(build_state_get_exit_nickname(circ->build_state)));
     rend_client_remove_intro_point(circ->build_state->chosen_exit,
@@ -690,7 +715,7 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
   }
 
   if (circ->n_conn)
-    connection_send_destroy(circ->n_circ_id, circ->n_conn);
+    connection_or_send_destroy(circ->n_circ_id, circ->n_conn, reason);
   for (conn=circ->n_streams; conn; conn=conn->next_stream)
     connection_edge_destroy(circ->n_circ_id, conn);
   while (circ->resolving_streams) {
@@ -699,23 +724,25 @@ _circuit_mark_for_close(circuit_t *circ, int line, const char *file)
     if (!conn->marked_for_close) {
       /* The other side will see a DESTROY, and infer that the connections
        * are closing because the circuit is getting torn down.  No need
-       * to send an end cell*/
-      conn->has_sent_end = 1; /* we're closing the circuit, nothing to send to */
+       * to send an end cell. */
+      conn->has_sent_end = 1;
       connection_mark_for_close(conn);
     }
     conn->on_circuit = NULL;
   }
   if (circ->p_conn)
-    connection_send_destroy(circ->p_circ_id, circ->p_conn);
+    connection_or_send_destroy(circ->p_circ_id, circ->p_conn, reason);
   for (conn=circ->p_streams; conn; conn=conn->next_stream)
     connection_edge_destroy(circ->p_circ_id, conn);
 
   circ->marked_for_close = line;
   circ->marked_for_close_file = file;
 
-  if (circ->rend_splice && !circ->rend_splice->marked_for_close) {
-    /* do this after marking this circuit, to avoid infinite recursion. */
-    circuit_mark_for_close(circ->rend_splice);
+  if (circ->rend_splice) {
+    if (!circ->rend_splice->marked_for_close) {
+      /* do this after marking this circuit, to avoid infinite recursion. */
+      circuit_mark_for_close(circ->rend_splice, reason);
+    }
     circ->rend_splice = NULL;
   }
 }
@@ -788,7 +815,8 @@ assert_circuit_ok(const circuit_t *c)
 
   if (c->n_conn) {
     tor_assert(c->n_conn->type == CONN_TYPE_OR);
-    tor_assert(!memcmp(c->n_conn->identity_digest, c->n_conn_id_digest, DIGEST_LEN));
+    tor_assert(!memcmp(c->n_conn->identity_digest, c->n_conn_id_digest,
+                       DIGEST_LEN));
     if (c->n_circ_id)
       tor_assert(c == circuit_get_by_circid_orconn(c->n_circ_id, c->n_conn));
   }
