@@ -2,8 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: command.c,v 1.104 2005/12/10 09:36:25 arma Exp $ */
-const char command_c_id[] = "$Id: command.c,v 1.104 2005/12/10 09:36:25 arma Exp $";
+/* $Id: command.c,v 1.107 2006/01/05 21:23:03 nickm Exp $ */
+const char command_c_id[] =
+  "$Id: command.c,v 1.107 2006/01/05 21:23:03 nickm Exp $";
 
 /**
  * \file command.c
@@ -19,11 +20,11 @@ const char command_c_id[] = "$Id: command.c,v 1.104 2005/12/10 09:36:25 arma Exp
 #include "or.h"
 
 /** Keep statistics about how many of each type of cell we've received. */
-unsigned long stats_n_padding_cells_processed = 0;
-unsigned long stats_n_create_cells_processed = 0;
-unsigned long stats_n_created_cells_processed = 0;
-unsigned long stats_n_relay_cells_processed = 0;
-unsigned long stats_n_destroy_cells_processed = 0;
+uint64_t stats_n_padding_cells_processed = 0;
+uint64_t stats_n_create_cells_processed = 0;
+uint64_t stats_n_created_cells_processed = 0;
+uint64_t stats_n_relay_cells_processed = 0;
+uint64_t stats_n_destroy_cells_processed = 0;
 
 /* These are the main four functions for processing cells */
 static void command_process_create_cell(cell_t *cell, connection_t *conn);
@@ -81,7 +82,8 @@ command_process_cell(cell_t *cell, connection_t *conn)
 
   if (now > current_second) { /* the second has rolled over */
     /* print stats */
-    info(LD_OR,"At end of second: %d creates (%d ms), %d createds (%d ms), %d relays (%d ms), %d destroys (%d ms)",
+    info(LD_OR,"At end of second: %d creates (%d ms), %d createds (%d ms), "
+         "%d relays (%d ms), %d destroys (%d ms)",
          num_create, create_time/1000,
          num_created, created_time/1000,
          num_relay, relay_time/1000,
@@ -150,19 +152,22 @@ command_process_cell(cell_t *cell, connection_t *conn)
   }
 }
 
-/** Process a 'create' <b>cell</b> that just arrived from <b>conn</b>. Make a new circuit
- * with the p_circ_id specified in cell. Put the circuit in state
- * onionskin_pending, and pass the onionskin to the cpuworker. Circ will
- * get picked up again when the cpuworker finishes decrypting it.
+/** Process a 'create' <b>cell</b> that just arrived from <b>conn</b>. Make a
+ * new circuit with the p_circ_id specified in cell. Put the circuit in state
+ * onionskin_pending, and pass the onionskin to the cpuworker. Circ will get
+ * picked up again when the cpuworker finishes decrypting it.
  */
 static void
 command_process_create_cell(cell_t *cell, connection_t *conn)
 {
   circuit_t *circ;
+  int id_is_high;
 
   if (we_are_hibernating()) {
-    info(LD_OR,"Received create cell but we're shutting down. Sending back destroy.");
-    connection_send_destroy(cell->circ_id, conn);
+    info(LD_OR,"Received create cell but we're shutting down. Sending back "
+         "destroy.");
+    connection_or_send_destroy(cell->circ_id, conn,
+                               END_CIRC_REASON_HIBERNATING);
     return;
   }
 
@@ -172,12 +177,15 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
    * This can happen because Tor 0.0.9pre5 and earlier decide which
    * half to use based on nickname, and we now use identity keys.
    */
-  if ((cell->circ_id & (1<<15)) && conn->circ_id_type == CIRC_ID_TYPE_HIGHER) {
-    info(LD_OR, "Got a high circuit ID from %s (%d); switching to low circuit IDs.",
+  id_is_high = cell->circ_id & (1<<15);
+  if (id_is_high && conn->circ_id_type == CIRC_ID_TYPE_HIGHER) {
+    info(LD_OR, "Got a high circuit ID from %s (%d); switching to "
+         "low circuit IDs.",
          conn->nickname ? conn->nickname : "client", conn->s);
     conn->circ_id_type = CIRC_ID_TYPE_LOWER;
-  } else if (!(cell->circ_id & (1<<15)) && conn->circ_id_type == CIRC_ID_TYPE_LOWER) {
-    info(LD_OR, "Got a low circuit ID from %s (%d); switching to high circuit IDs.",
+  } else if (!id_is_high && conn->circ_id_type == CIRC_ID_TYPE_LOWER) {
+    info(LD_OR, "Got a low circuit ID from %s (%d); switching to "
+         "high circuit IDs.",
          conn->nickname ? conn->nickname : "client", conn->s);
     conn->circ_id_type = CIRC_ID_TYPE_HIGHER;
   }
@@ -187,7 +195,8 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
   if (circ) {
     routerinfo_t *router = router_get_by_digest(conn->identity_digest);
     log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
-           "received CREATE cell (circID %d) for known circ. Dropping (age %d).",
+           "received CREATE cell (circID %d) for known circ. "
+           "Dropping (age %d).",
            cell->circ_id, (int)(time(NULL) - conn->timestamp_created));
     if (router)
       log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
@@ -206,7 +215,7 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
     /* hand it off to the cpuworkers, and then return */
     if (assign_to_cpuworker(NULL, CPUWORKER_TASK_ONION, circ) < 0) {
       warn(LD_GENERAL,"Failed to hand off onionskin. Closing.");
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, END_CIRC_REASON_INTERNAL);
       return;
     }
     debug(LD_OR,"success: handed off onionskin.");
@@ -218,12 +227,12 @@ command_process_create_cell(cell_t *cell, connection_t *conn)
     tor_assert(cell->command == CELL_CREATE_FAST);
     if (fast_server_handshake(cell->payload, reply, keys, sizeof(keys))<0) {
       warn(LD_OR,"Failed to generate key material. Closing.");
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, END_CIRC_REASON_INTERNAL);
       return;
     }
     if (onionskin_answer(circ, CELL_CREATED_FAST, reply, keys)<0) {
       warn(LD_OR,"Failed to reply to CREATE_FAST cell. Closing.");
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, END_CIRC_REASON_INTERNAL);
       return;
     }
   }
@@ -245,14 +254,15 @@ command_process_created_cell(cell_t *cell, connection_t *conn)
   circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
   if (!circ) {
-    info(LD_OR,"(circID %d) unknown circ (probably got a destroy earlier). Dropping.", cell->circ_id);
+    info(LD_OR,"(circID %d) unknown circ (probably got a destroy earlier). "
+         "Dropping.", cell->circ_id);
     return;
   }
 
   if (circ->n_circ_id != cell->circ_id) {
     log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,
            "got created cell from OPward? Closing.");
-    circuit_mark_for_close(circ);
+    circuit_mark_for_close(circ, END_CIRC_REASON_TORPROTOCOL);
     return;
   }
 
@@ -260,13 +270,14 @@ command_process_created_cell(cell_t *cell, connection_t *conn)
     debug(LD_OR,"at OP. Finishing handshake.");
     if (circuit_finish_handshake(circ, cell->command, cell->payload) < 0) {
       warn(LD_OR,"circuit_finish_handshake failed.");
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, END_CIRC_AT_ORIGIN);
       return;
     }
     debug(LD_OR,"Moving to next skin.");
     if (circuit_send_next_onion_skin(circ) < 0) {
       info(LD_OR,"circuit_send_next_onion_skin failed.");
-      circuit_mark_for_close(circ); /* XXX push this circuit_close lower */
+      /* XXX push this circuit_close lower */
+      circuit_mark_for_close(circ, END_CIRC_AT_ORIGIN);
       return;
     }
   } else { /* pack it into an extended relay cell, and send it. */
@@ -284,6 +295,7 @@ static void
 command_process_relay_cell(cell_t *cell, connection_t *conn)
 {
   circuit_t *circ;
+  int reason;
 
   circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
 
@@ -295,20 +307,24 @@ command_process_relay_cell(cell_t *cell, connection_t *conn)
 
   if (circ->state == CIRCUIT_STATE_ONIONSKIN_PENDING) {
     log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,"circuit in create_wait. Closing.");
-    circuit_mark_for_close(circ);
+    circuit_mark_for_close(circ, END_CIRC_REASON_TORPROTOCOL);
     return;
   }
 
   if (cell->circ_id == circ->p_circ_id) { /* it's an outgoing cell */
-    if (circuit_receive_relay_cell(cell, circ, CELL_DIRECTION_OUT) < 0) {
-      log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,"circuit_receive_relay_cell (forward) failed. Closing.");
-      circuit_mark_for_close(circ);
+    if ((reason = circuit_receive_relay_cell(cell, circ,
+                                             CELL_DIRECTION_OUT)) < 0) {
+      log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,"circuit_receive_relay_cell "
+             "(forward) failed. Closing.");
+      circuit_mark_for_close(circ, -reason);
       return;
     }
   } else { /* it's an ingoing cell */
-    if (circuit_receive_relay_cell(cell, circ, CELL_DIRECTION_IN) < 0) {
-      log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,"circuit_receive_relay_cell (backward) failed. Closing.");
-      circuit_mark_for_close(circ);
+    if ((reason = circuit_receive_relay_cell(cell, circ,
+                                             CELL_DIRECTION_IN)) < 0) {
+      log_fn(LOG_PROTOCOL_WARN,LD_PROTOCOL,"circuit_receive_relay_cell "
+             "(backward) failed. Closing.");
+      circuit_mark_for_close(circ, -reason);
       return;
     }
   }
@@ -331,9 +347,10 @@ static void
 command_process_destroy_cell(cell_t *cell, connection_t *conn)
 {
   circuit_t *circ;
+  uint8_t reason;
 
   circ = circuit_get_by_circid_orconn(cell->circ_id, conn);
-
+  reason = (uint8_t)cell->payload[0];
   if (!circ) {
     info(LD_OR,"unknown circuit %d on connection from %s:%d. Dropping.",
          cell->circ_id, conn->address, conn->port);
@@ -344,15 +361,17 @@ command_process_destroy_cell(cell_t *cell, connection_t *conn)
   if (cell->circ_id == circ->p_circ_id) {
     /* the destroy came from behind */
     circuit_set_circid_orconn(circ, 0, NULL, P_CONN_CHANGED);
-    circuit_mark_for_close(circ);
+    circuit_mark_for_close(circ, reason);
   } else { /* the destroy came from ahead */
     circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
     if (CIRCUIT_IS_ORIGIN(circ)) {
-      circuit_mark_for_close(circ);
+      circuit_mark_for_close(circ, reason);
     } else {
+      char payload[1];
       debug(LD_OR, "Delivering 'truncated' back.");
+      payload[0] = (char)reason;
       connection_edge_send_command(NULL, circ, RELAY_COMMAND_TRUNCATED,
-                                   NULL, 0, NULL);
+                                   payload, sizeof(payload), NULL);
     }
   }
 }
