@@ -2,8 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: connection_or.c,v 1.201 2005/12/10 09:36:25 arma Exp $ */
-const char connection_or_c_id[] = "$Id: connection_or.c,v 1.201 2005/12/10 09:36:25 arma Exp $";
+/* $Id: connection_or.c,v 1.209 2006/01/10 22:42:44 arma Exp $ */
+const char connection_or_c_id[] =
+  "$Id: connection_or.c,v 1.209 2006/01/10 22:42:44 arma Exp $";
 
 /**
  * \file connection_or.c
@@ -76,8 +77,10 @@ connection_or_clear_identity_map(void)
     }
   }
 
-  digestmap_free(orconn_identity_map, NULL);
-  orconn_identity_map = NULL;
+  if (orconn_identity_map) {
+    digestmap_free(orconn_identity_map, NULL);
+    orconn_identity_map = NULL;
+  }
 }
 
 /** Change conn->identity_digest to digest, and add conn into
@@ -157,7 +160,8 @@ connection_or_read_proxy_response(connection_t *conn)
                               &headers, MAX_HEADERS_SIZE,
                               NULL, NULL, 10000, 0)) {
     case -1: /* overflow */
-      warn(LD_PROTOCOL,"Your https proxy sent back an oversized response. Closing.");
+      warn(LD_PROTOCOL,"Your https proxy sent back an oversized response. "
+           "Closing.");
       return -1;
     case 0:
       info(LD_OR,"https proxy response not all here yet. Waiting.");
@@ -189,7 +193,8 @@ connection_or_read_proxy_response(connection_t *conn)
   }
   /* else, bad news on the status code */
   warn(LD_OR,
-       "The https proxy sent back an unexpected status code %d (\"%s\"). Closing.",
+       "The https proxy sent back an unexpected status code %d (\"%s\"). "
+       "Closing.",
        status_code, reason);
   tor_free(reason);
   connection_mark_for_close(conn);
@@ -442,9 +447,13 @@ connection_or_connect(uint32_t addr, uint16_t port, const char *id_digest)
 
   switch (connection_connect(conn, conn->address, addr, port)) {
     case -1:
-      if (!options->HttpsProxy)
+      /* If the connection failed immediately, and we're using
+       * an https proxy, our https proxy is down. Don't blame the
+       * Tor server. */
+      if (!options->HttpsProxy) {
         router_mark_as_down(conn->identity_digest);
-      helper_node_set_status(conn->identity_digest, 0);
+        entry_guard_set_status(conn->identity_digest, 0);
+      }
       control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
       connection_free(conn);
       return NULL;
@@ -466,8 +475,8 @@ connection_or_connect(uint32_t addr, uint16_t port, const char *id_digest)
 /** Begin the tls handshake with <b>conn</b>. <b>receiving</b> is 0 if
  * we initiated the connection, else it's 1.
  *
- * Assign a new tls object to conn->tls, begin reading on <b>conn</b>, and pass
- * <b>conn</b> to connection_tls_continue_handshake().
+ * Assign a new tls object to conn->tls, begin reading on <b>conn</b>, and
+ * pass <b>conn</b> to connection_tls_continue_handshake().
  *
  * Return -1 if <b>conn</b> is broken, else return 0.
  */
@@ -566,7 +575,8 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
   }
   check_no_tls_errors();
   if (tor_tls_get_peer_cert_nickname(conn->tls, nickname, sizeof(nickname))) {
-    log_fn(severity,LD_PROTOCOL,"Other side (%s:%d) has a cert without a valid nickname. Closing.",
+    log_fn(severity,LD_PROTOCOL,"Other side (%s:%d) has a cert without a "
+           "valid nickname. Closing.",
            conn->address, conn->port);
     return -1;
   }
@@ -575,7 +585,8 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
         conn->address, conn->port, nickname);
 
   if (tor_tls_verify(severity, conn->tls, &identity_rcvd) < 0) {
-    log_fn(severity,LD_OR,"Other side, which claims to be router '%s' (%s:%d), has a cert but it's invalid. Closing.",
+    log_fn(severity,LD_OR,"Other side, which claims to be router '%s' (%s:%d),"
+           " has a cert but it's invalid. Closing.",
            nickname, conn->address, conn->port);
     return -1;
   }
@@ -595,7 +606,8 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
       router->is_named && /* make sure it's the right guy */
       memcmp(digest_rcvd, router->cache_info.identity_digest,DIGEST_LEN) !=0) {
     log_fn(severity, LD_OR,
-           "Identity key not as expected for router claiming to be '%s' (%s:%d)",
+           "Identity key not as expected for router claiming to be "
+           "'%s' (%s:%d)",
            nickname, conn->address, conn->port);
     return -1;
   }
@@ -607,11 +619,13 @@ connection_or_check_valid_handshake(connection_t *conn, char *digest_rcvd)
       char seen[HEX_DIGEST_LEN+1];
       char expected[HEX_DIGEST_LEN+1];
       base16_encode(seen, sizeof(seen), digest_rcvd, DIGEST_LEN);
-      base16_encode(expected, sizeof(expected), conn->identity_digest, DIGEST_LEN);
+      base16_encode(expected, sizeof(expected), conn->identity_digest,
+                    DIGEST_LEN);
       log_fn(severity, LD_OR,
-             "Identity key not as expected for router at %s:%d: wanted %s but got %s",
+             "Identity key not as expected for router at %s:%d: wanted %s "
+             "but got %s",
              conn->address, conn->port, expected, seen);
-      helper_node_set_status(conn->identity_digest, 0);
+      entry_guard_set_status(conn->identity_digest, 0);
       control_event_or_conn_status(conn, OR_CONN_EVENT_FAILED);
       as_advertised = 0;
     }
@@ -645,12 +659,13 @@ static int
 connection_tls_finish_handshake(connection_t *conn)
 {
   char digest_rcvd[DIGEST_LEN];
+  int started_here = connection_or_nonopen_was_started_here(conn);
 
   debug(LD_OR,"tls handshake done. verifying.");
   if (connection_or_check_valid_handshake(conn, digest_rcvd) < 0)
     return -1;
 
-  if (!connection_or_nonopen_was_started_here(conn)) {
+  if (!started_here) {
 #if 0
     connection_t *c;
     if ((c=connection_or_get_by_identity_digest(digest_rcvd))) {
@@ -659,7 +674,8 @@ connection_tls_finish_handshake(connection_t *conn)
       return -1;
     }
 #endif
-    connection_or_init_conn_from_address(conn,conn->addr,conn->port,digest_rcvd);
+    connection_or_init_conn_from_address(conn,conn->addr,conn->port,
+                                         digest_rcvd);
   }
 
   if (!server_mode(get_options())) { /* If I'm an OP... */
@@ -668,18 +684,23 @@ connection_tls_finish_handshake(connection_t *conn)
 
   directory_set_dirty();
   conn->state = OR_CONN_STATE_OPEN;
+  control_event_or_conn_status(conn, OR_CONN_EVENT_CONNECTED);
+  if (started_here) {
+    rep_hist_note_connect_succeeded(conn->identity_digest, time(NULL));
+    if (entry_guard_set_status(conn->identity_digest, 1) < 0) {
+      /* pending circs get closed in circuit_about_to_close_connection() */
+      return -1;
+    }
+  }
   connection_watch_events(conn, EV_READ);
   circuit_n_conn_done(conn, 1); /* send the pending creates, if any. */
-  rep_hist_note_connect_succeeded(conn->identity_digest, time(NULL));
-  helper_node_set_status(conn->identity_digest, 1);
-  control_event_or_conn_status(conn, OR_CONN_EVENT_CONNECTED);
   return 0;
 }
 
 /** Pack <b>cell</b> into wire-format, and write it onto <b>conn</b>'s
  * outbuf.
  *
- * (Commented out) If it's an OR conn, and an entire TLS record is
+ * If it's an OR conn, and an entire TLS record is
  * ready, then try to flush the record now.
  */
 void
@@ -709,10 +730,11 @@ connection_or_write_cell_to_buf(const cell_t *cell, connection_t *conn)
     if (connection_handle_write(conn) < 0) {
       if (!conn->marked_for_close) {
         /* this connection is broken. remove it. */
-        warn(LD_BUG,"Bug: unhandled error on write for OR conn (fd %d); removing",
+        warn(LD_BUG,
+             "Bug: unhandled error on write for OR conn (fd %d); removing",
              conn->s);
         tor_fragile_assert();
-        conn->has_sent_end = 1; /* otherwise we cry wolf about duplicate close */
+        conn->has_sent_end = 1; /* don't cry wolf about duplicate close */
         /* XXX do we need a close-immediate here, so we don't try to flush? */
         connection_mark_for_close(conn);
       }
@@ -740,8 +762,10 @@ connection_or_process_cells_from_inbuf(connection_t *conn)
 
 loop:
   debug(LD_OR,"%d: starting, inbuf_datalen %d (%d pending in tls object).",
-        conn->s,(int)buf_datalen(conn->inbuf),tor_tls_get_pending_bytes(conn->tls));
-  if (buf_datalen(conn->inbuf) < CELL_NETWORK_SIZE) /* entire response available? */
+        conn->s,(int)buf_datalen(conn->inbuf),
+        tor_tls_get_pending_bytes(conn->tls));
+  if (buf_datalen(conn->inbuf) < CELL_NETWORK_SIZE) /* whole response
+                                                       available? */
     return 0; /* not yet */
 
   connection_fetch_from_buf(buf, CELL_NETWORK_SIZE, conn);
@@ -755,3 +779,26 @@ loop:
   goto loop; /* process the remainder of the buffer */
 }
 
+/** Write a destroy cell with circ ID <b>circ_id</b> and reason <b>reason</b>
+ * onto OR connection <b>conn</b>.  Don't perform range-checking on reason:
+ * we may want to propagate reasons from other cells.
+ *
+ * Return 0.
+ */
+int
+connection_or_send_destroy(uint16_t circ_id, connection_t *conn, int reason)
+{
+  cell_t cell;
+
+  tor_assert(conn);
+  tor_assert(connection_speaks_cells(conn));
+
+  memset(&cell, 0, sizeof(cell_t));
+  cell.circ_id = circ_id;
+  cell.command = CELL_DESTROY;
+  cell.payload[0] = (uint8_t) reason;
+  debug(LD_OR,"Sending destroy (circID %d).", circ_id);
+  connection_or_write_cell_to_buf(&cell, conn);
+  return 0;
+}
+
