@@ -1,8 +1,9 @@
 /* Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: directory.c,v 1.324 2005/12/10 09:36:25 arma Exp $ */
-const char directory_c_id[] = "$Id: directory.c,v 1.324 2005/12/10 09:36:25 arma Exp $";
+/* $Id: directory.c,v 1.341 2006/01/10 20:50:22 nickm Exp $ */
+const char directory_c_id[] =
+  "$Id: directory.c,v 1.341 2006/01/10 20:50:22 nickm Exp $";
 
 #include "or.h"
 
@@ -30,11 +31,6 @@ const char directory_c_id[] = "$Id: directory.c,v 1.324 2005/12/10 09:36:25 arma
  *   connection_finished_connecting() in connection.c
  */
 static void
-directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
-                                      uint8_t purpose, int private_connection,
-                                      const char *resource,
-                                      const char *payload, size_t payload_len);
-static void
 directory_initiate_command(const char *address, uint32_t addr, uint16_t port,
                            const char *platform,
                            const char *digest, uint8_t purpose,
@@ -54,6 +50,7 @@ static void connection_dir_download_networkstatus_failed(connection_t *conn);
 static void connection_dir_download_routerdesc_failed(connection_t *conn);
 static void dir_networkstatus_download_failed(smartlist_t *failed);
 static void dir_routerdesc_download_failed(smartlist_t *failed);
+static void note_request(const char *key, size_t bytes);
 
 /********* START VARIABLES **********/
 
@@ -142,10 +139,11 @@ directory_post_to_dirservers(uint8_t purpose, const char *payload,
    */
   SMARTLIST_FOREACH(dirservers, trusted_dir_server_t *, ds,
     {
+      routerstatus_t *rs = &(ds->fake_status);
       post_via_tor = purpose_is_private(purpose) ||
                      !fascist_firewall_allows_address(ds->addr,ds->dir_port);
-      directory_initiate_command_trusted_dir(ds, purpose, post_via_tor,
-                                             NULL, payload, payload_len);
+      directory_initiate_command_routerstatus(rs, purpose, post_via_tor,
+                                              NULL, payload, payload_len);
     });
 }
 
@@ -158,10 +156,9 @@ void
 directory_get_from_dirserver(uint8_t purpose, const char *resource,
                              int retry_if_no_servers)
 {
-  routerinfo_t *r = NULL;
-  trusted_dir_server_t *ds = NULL;
+  routerstatus_t *rs = NULL;
   or_options_t *options = get_options();
-  int fetch_fresh_first = server_mode(options) && options->DirPort != 0;
+  int prefer_authority = server_mode(options) && options->DirPort != 0;
   int directconn = !purpose_is_private(purpose);
 
   int need_v1_support = purpose == DIR_PURPOSE_FETCH_DIR ||
@@ -170,23 +167,16 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
                         purpose == DIR_PURPOSE_FETCH_SERVERDESC;
 
   if (directconn) {
-    if (fetch_fresh_first && purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS &&
-        !strcmpstart(resource,"fp/") && strlen(resource) == HEX_DIGEST_LEN+3) {
-      /* Try to ask the actual dirserver its opinion. */
-      char digest[DIGEST_LEN];
-      base16_decode(digest, DIGEST_LEN, resource+3, HEX_DIGEST_LEN);
-      ds = router_get_trusteddirserver_by_digest(digest);
-    }
-    if (!ds && fetch_fresh_first) {
+    if (prefer_authority) {
       /* only ask authdirservers, and don't ask myself */
-      ds = router_pick_trusteddirserver(need_v1_support, 1, 1,
+      rs = router_pick_trusteddirserver(need_v1_support, 1, 1,
                                         retry_if_no_servers);
     }
-    if (!ds) {
+    if (!rs) {
       /* anybody with a non-zero dirport will do */
-      r = router_pick_directory_server(1, 1, need_v2_support,
-                                       retry_if_no_servers);
-      if (!r) {
+      rs = router_pick_directory_server(1, 1, need_v2_support,
+                                        retry_if_no_servers);
+      if (!rs) {
         const char *which;
         if (purpose == DIR_PURPOSE_FETCH_DIR)
           which = "directory";
@@ -198,9 +188,9 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
           which = "server descriptors";
         info(LD_DIR,
              "No router found for %s; falling back to dirserver list", which);
-        ds = router_pick_trusteddirserver(1, 1, 1,
+        rs = router_pick_trusteddirserver(1, 1, 1,
                                           retry_if_no_servers);
-        if (!ds)
+        if (!rs)
           directconn = 0; /* last resort: try routing it via Tor */
       }
     }
@@ -209,20 +199,20 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
     /* Never use fascistfirewall; we're going via Tor. */
     if (purpose == DIR_PURPOSE_FETCH_RENDDESC) {
       /* only ask authdirservers, any of them will do */
-      ds = router_pick_trusteddirserver(0, 0, 0, retry_if_no_servers);
+      rs = router_pick_trusteddirserver(0, 0, 0, retry_if_no_servers);
     } else {
       /* anybody with a non-zero dirport will do. Disregard firewalls. */
-      r = router_pick_directory_server(1, 0, need_v2_support,
-                                       retry_if_no_servers);
+      rs = router_pick_directory_server(1, 0, need_v2_support,
+                                        retry_if_no_servers);
+      /* If we have any hope of building an indirect conn, we know some router
+       * descriptors.  If (rs==NULL), we can't build circuits anyway, so
+       * there's no point in falling back to the authorities in this case.  */
     }
   }
 
-  if (r)
-    directory_initiate_command_router(r, purpose, !directconn,
-                                      resource, NULL, 0);
-  else if (ds)
-    directory_initiate_command_trusted_dir(ds, purpose, !directconn,
-                                           resource, NULL, 0);
+  if (rs)
+    directory_initiate_command_routerstatus(rs, purpose, !directconn,
+                                            resource, NULL, 0);
   else {
     notice(LD_DIR,
            "No running dirservers known. Will try again later. (purpose %d)",
@@ -246,28 +236,56 @@ directory_get_from_dirserver(uint8_t purpose, const char *resource,
  * want to fetch.
  */
 void
-directory_initiate_command_router(routerinfo_t *router, uint8_t purpose,
-                                  int private_connection, const char *resource,
-                                  const char *payload, size_t payload_len)
+directory_initiate_command_router(routerinfo_t *router,
+                                  uint8_t purpose,
+                                  int private_connection,
+                                  const char *resource,
+                                  const char *payload,
+                                  size_t payload_len)
 {
   directory_initiate_command(router->address, router->addr, router->dir_port,
-                         router->platform, router->cache_info.identity_digest,
-                         purpose, private_connection, resource,
-                         payload, payload_len);
+                             router->platform,
+                             router->cache_info.identity_digest,
+                             purpose, private_connection, resource,
+                             payload, payload_len);
 }
 
-/** As directory_initiate_command_router, but send the command to a trusted
- * directory server <b>dirserv</b>. **/
-static void
-directory_initiate_command_trusted_dir(trusted_dir_server_t *dirserv,
-                                       uint8_t purpose, int private_connection,
-                                       const char *resource,
-                                       const char *payload, size_t payload_len)
+/** Launch a new connection to the directory server <b>status</b> to upload or
+ * download a service or rendezvous descriptor. <b>purpose</b> determines what
+ * kind of directory connection we're launching, and must be one of
+ * DIR_PURPOSE_{FETCH|UPLOAD}_{DIR|RENDDESC}.
+ *
+ * When uploading, <b>payload</b> and <b>payload_len</b> determine the content
+ * of the HTTP post.  Otherwise, <b>payload</b> should be NULL.
+ *
+ * When fetching a rendezvous descriptor, <b>resource</b> is the service ID we
+ * want to fetch.
+ */
+void
+directory_initiate_command_routerstatus(routerstatus_t *status,
+                                        uint8_t purpose,
+                                        int private_connection,
+                                        const char *resource,
+                                        const char *payload,
+                                        size_t payload_len)
 {
-  directory_initiate_command(dirserv->address, dirserv->addr,
-               dirserv->dir_port, NULL, dirserv->digest, purpose,
-               private_connection, resource,
-               payload, payload_len);
+  const char *platform = NULL;
+  routerinfo_t *router;
+  char address_buf[INET_NTOA_BUF_LEN];
+  struct in_addr in;
+  const char *address;
+  if ((router = router_get_by_digest(status->identity_digest))) {
+    platform = router->platform;
+    address = router->address;
+  } else {
+    in.s_addr = htonl(status->addr);
+    tor_inet_ntoa(&in, address_buf, sizeof(address_buf));
+    address = address_buf;
+  }
+  directory_initiate_command(address, status->addr, status->dir_port,
+                             platform, status->identity_digest,
+                             purpose, private_connection, resource,
+                             payload, payload_len);
 }
 
 /** Called when we are unable to complete the client's request to a
@@ -433,8 +451,12 @@ directory_initiate_command(const char *address, uint32_t addr,
       return;
     }
 
+    if (connection_add(conn) < 0) {
+      warn(LD_NET,"Unable to add AP bridge to dirserver.");
+      connection_mark_for_close(conn);
+      return;
+    }
     conn->state = DIR_CONN_STATE_CLIENT_SENDING;
-    connection_add(conn);
     /* queue the command on the outbuf */
     directory_send_command(conn, platform, purpose, resource,
                            payload, payload_len);
@@ -469,7 +491,8 @@ directory_send_command(connection_t *conn, const char *platform,
   if (conn->port == 80) {
     strlcpy(hoststring, conn->address, sizeof(hoststring));
   } else {
-    tor_snprintf(hoststring, sizeof(hoststring),"%s:%d",conn->address, conn->port);
+    tor_snprintf(hoststring, sizeof(hoststring),"%s:%d",
+                 conn->address, conn->port);
   }
 
   /* come up with some proxy lines, if we're using one. */
@@ -561,10 +584,18 @@ directory_send_command(connection_t *conn, const char *platform,
   connection_write_to_buf(url, strlen(url), conn);
   tor_free(url);
 
-  tor_snprintf(request, sizeof(request), " HTTP/1.0\r\nContent-Length: %lu\r\nHost: %s%s\r\n\r\n",
-           payload ? (unsigned long)payload_len : 0,
-           hoststring,
-           proxyauthstring);
+  if (!strcmp(httpcommand, "GET") && !payload) {
+    tor_snprintf(request, sizeof(request),
+                 " HTTP/1.0\r\nHost: %s%s\r\n\r\n",
+                 hoststring,
+                 proxyauthstring);
+  } else {
+    tor_snprintf(request, sizeof(request),
+                 " HTTP/1.0\r\nContent-Length: %lu\r\nHost: %s%s\r\n\r\n",
+                 payload ? (unsigned long)payload_len : 0,
+                 hoststring,
+                 proxyauthstring);
+  }
   connection_write_to_buf(request, strlen(request), conn);
 
   if (payload) {
@@ -749,7 +780,8 @@ parse_http_response(const char *headers, int *code, time_t *date,
     } else if (!strcmp(enc, "gzip") || !strcmp(enc, "x-gzip")) {
       *compression = GZIP_METHOD;
     } else {
-      info(LD_HTTP, "Unrecognized content encoding: '%s'. Trying to deal.", enc);
+      info(LD_HTTP, "Unrecognized content encoding: '%s'. Trying to deal.",
+           enc);
       *compression = -1;
     }
   }
@@ -789,9 +821,9 @@ body_is_plausible(const char *body, size_t len, int purpose)
 /** We are a client, and we've finished reading the server's
  * response. Parse and it and act appropriately.
  *
- * If we're happy with the result (we get it and it's useful),
- * return 0. Otherwise return -1, and the caller should consider
- * trying the request again.
+ * If we're still happy with using this directory server in the future, return
+ * 0. Otherwise return -1; and the caller should consider trying the request
+ * again.
  *
  * The caller will take care of marking the connection for close.
  */
@@ -801,7 +833,7 @@ connection_dir_client_reached_eof(connection_t *conn)
   char *body;
   char *headers;
   char *reason = NULL;
-  size_t body_len=0;
+  size_t body_len=0, orig_len=0;
   int status_code;
   time_t now, date_header=0;
   int delta;
@@ -809,6 +841,7 @@ connection_dir_client_reached_eof(connection_t *conn)
   int plausible;
   int skewed=0;
   int allow_partial = conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC;
+  int was_compressed=0;
 
   switch (fetch_from_buf_http(conn->inbuf,
                               &headers, MAX_HEADERS_SIZE,
@@ -819,10 +852,12 @@ connection_dir_client_reached_eof(connection_t *conn)
            conn->address, conn->port);
       return -1;
     case 0:
-      info(LD_HTTP,"'fetch' response not all here, but we're at eof. Closing.");
+      info(LD_HTTP,
+           "'fetch' response not all here, but we're at eof. Closing.");
       return -1;
     /* case 1, fall through */
   }
+  orig_len = body_len;
 
   if (parse_http_response(headers, &status_code, &date_header,
                           &compression, &reason) < 0) {
@@ -841,15 +876,18 @@ connection_dir_client_reached_eof(connection_t *conn)
     now = time(NULL);
     delta = now-date_header;
     if (abs(delta)>ALLOW_DIRECTORY_TIME_SKEW) {
-      log_fn(router_digest_is_trusted_dir(conn->identity_digest) ? LOG_WARN : LOG_INFO,
+      log_fn(router_digest_is_trusted_dir(conn->identity_digest) ?
+                                                        LOG_WARN : LOG_INFO,
              LD_HTTP,
-             "Received directory with skewed time (server '%s:%d'): we are %d minutes %s, or the directory is %d minutes %s.",
+             "Received directory with skewed time (server '%s:%d'): "
+             "we are %d minutes %s, or the directory is %d minutes %s.",
              conn->address, conn->port,
              abs(delta)/60, delta>0 ? "ahead" : "behind",
              abs(delta)/60, delta>0 ? "behind" : "ahead");
       skewed = 1; /* don't check the recommended-versions line */
     } else {
-      debug(LD_HTTP, "Time on received directory is within tolerance; we are %d seconds skewed.  (That's okay.)", delta);
+      debug(LD_HTTP, "Time on received directory is within tolerance; "
+            "we are %d seconds skewed.  (That's okay.)", delta);
     }
   }
 
@@ -904,11 +942,12 @@ connection_dir_client_reached_eof(connection_t *conn)
       tor_free(body);
       body = new_body;
       body_len = new_len;
+      was_compressed = 1;
     }
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_DIR) {
-    /* fetch/process the directory to learn about new routers. */
+    /* fetch/process the directory to cache it. */
     info(LD_DIR,"Received directory (size %d) from server '%s:%d'",
          (int)body_len, conn->address, conn->port);
     if (status_code == 503 || body_len == 0) {
@@ -918,45 +957,55 @@ connection_dir_client_reached_eof(connection_t *conn)
       return -1;
     }
     if (status_code != 200) {
-      warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
+      warn(LD_DIR,"Received http status code %d (\"%s\") from server "
+           "'%s:%d'. I'll try again soon.",
            status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (router_parse_directory(body) < 0) {
-      notice(LD_DIR,"I failed to parse the directory I fetched from '%s:%d'. Ignoring.", conn->address, conn->port);
+      notice(LD_DIR,"I failed to parse the directory I fetched from "
+             "'%s:%d'. Ignoring.", conn->address, conn->port);
     }
+    note_request(was_compressed?"dl/dir.z":"dl/dir", orig_len);
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RUNNING_LIST) {
     /* just update our list of running routers, if this list is new info */
     info(LD_DIR,"Received running-routers list (size %d)", (int)body_len);
     if (status_code != 200) {
-      warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d'. I'll try again soon.",
+      warn(LD_DIR,"Received http status code %d (\"%s\") from server "
+           "'%s:%d'. I'll try again soon.",
            status_code, reason, conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
     if (router_parse_runningrouters(body)<0) {
-      warn(LD_DIR,"Bad running-routers from server '%s:%d'. I'll try again soon.",
+      warn(LD_DIR,"Bad running-routers from server '%s:%d'. I'll try again "
+           "soon.",
            conn->address, conn->port);
       tor_free(body); tor_free(headers); tor_free(reason);
       return -1;
     }
+    note_request(was_compressed?"dl/running-routers.z":
+                 "dl/running-routers", orig_len);
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_NETWORKSTATUS) {
     smartlist_t *which = NULL;
     char *cp;
-    info(LD_DIR,"Received networkstatus objects (size %d) from server '%s:%d'",(int) body_len, conn->address, conn->port);
+    info(LD_DIR,"Received networkstatus objects (size %d) from server "
+         "'%s:%d'",(int) body_len, conn->address, conn->port);
     if (status_code != 200) {
-      warn(LD_DIR,"Received http status code %d (\"%s\") from server '%s:%d' while fetching \"/tor/status/%s\". I'll try again soon.",
+      warn(LD_DIR,"Received http status code %d (\"%s\") from server "
+           "'%s:%d' while fetching \"/tor/status/%s\". I'll try again soon.",
            status_code, reason, conn->address, conn->port,
            conn->requested_resource);
       tor_free(body); tor_free(headers); tor_free(reason);
       connection_dir_download_networkstatus_failed(conn);
       return -1;
     }
+    note_request(was_compressed?"dl/status.z":"dl/status", orig_len);
     if (conn->requested_resource &&
         !strcmpstart(conn->requested_resource,"fp/")) {
       which = smartlist_create();
@@ -994,20 +1043,22 @@ connection_dir_client_reached_eof(connection_t *conn)
     int n_asked_for = 0;
     info(LD_DIR,"Received server info (size %d) from server '%s:%d'",
          (int)body_len, conn->address, conn->port);
+    note_request(was_compressed?"dl/server.z":"dl/server", orig_len);
     if (conn->requested_resource &&
-        !strcmpstart(conn->requested_resource,"fp/")) {
+        !strcmpstart(conn->requested_resource,"d/")) {
       which = smartlist_create();
-      dir_split_resource_into_fingerprints(conn->requested_resource+3,
+      dir_split_resource_into_fingerprints(conn->requested_resource+2,
                                            which, NULL, 0);
       n_asked_for = smartlist_len(which);
     }
     if (status_code != 200) {
-      int no_warn = status_code == 404 ||
+      int dir_okay = status_code == 404 ||
         (status_code == 400 && !strcmp(reason, "Servers unavailable."));
       /* 404 means that it didn't have them; no big deal.
        * Older (pre-0.1.1.8) servers said 400 Servers unavailable instead. */
-      log_fn(no_warn ? LOG_INFO : LOG_WARN, LD_DIR,
-             "Received http status code %d (\"%s\") from server '%s:%d' while fetching \"/tor/server/%s\". I'll try again soon.",
+      log_fn(dir_okay ? LOG_INFO : LOG_WARN, LD_DIR,
+             "Received http status code %d (\"%s\") from server '%s:%d' "
+             "while fetching \"/tor/server/%s\". I'll try again soon.",
              status_code, reason, conn->address, conn->port,
              conn->requested_resource);
       if (!which) {
@@ -1018,7 +1069,7 @@ connection_dir_client_reached_eof(connection_t *conn)
         smartlist_free(which);
       }
       tor_free(body); tor_free(headers); tor_free(reason);
-      return -1;
+      return dir_okay ? 0 : -1;
     }
     /* Learn the routers, assuming we requested by fingerprint or "all".
      * Right now, we only use "authority" to fetch ourself, so we don't want
@@ -1056,10 +1107,13 @@ connection_dir_client_reached_eof(connection_t *conn)
   if (conn->purpose == DIR_PURPOSE_UPLOAD_DIR) {
     switch (status_code) {
       case 200:
-        info(LD_GENERAL,"eof (status 200) after uploading server descriptor: finished.");
+        info(LD_GENERAL,"eof (status 200) after uploading server "
+             "descriptor: finished.");
         break;
       case 400:
-        warn(LD_GENERAL,"http status 400 (\"%s\") response from dirserver '%s:%d'. Please correct.", reason, conn->address, conn->port);
+        warn(LD_GENERAL,"http status 400 (\"%s\") response from "
+             "dirserver '%s:%d'. Please correct.",
+             reason, conn->address, conn->port);
         break;
       case 403:
         warn(LD_GENERAL,"http status 403 (\"%s\") response from dirserver "
@@ -1069,7 +1123,8 @@ connection_dir_client_reached_eof(connection_t *conn)
              "tor-doc-server.html", reason, conn->address, conn->port);
         break;
       default:
-        warn(LD_GENERAL,"http status %d (\"%s\") reason unexpected (server '%s:%d').",
+        warn(LD_GENERAL,"http status %d (\"%s\") reason unexpected (server "
+             "'%s:%d').",
              status_code, reason, conn->address, conn->port);
         break;
     }
@@ -1078,7 +1133,8 @@ connection_dir_client_reached_eof(connection_t *conn)
   }
 
   if (conn->purpose == DIR_PURPOSE_FETCH_RENDDESC) {
-    info(LD_REND,"Received rendezvous descriptor (size %d, status %d (\"%s\"))",
+    info(LD_REND,"Received rendezvous descriptor (size %d, status %d "
+         "(\"%s\"))",
          (int)body_len, status_code, reason);
     switch (status_code) {
       case 200:
@@ -1097,10 +1153,12 @@ connection_dir_client_reached_eof(connection_t *conn)
          * connection_mark_for_close cleans it up. */
         break;
       case 400:
-        warn(LD_REND,"http status 400 (\"%s\"). Dirserver didn't like our rendezvous query?", reason);
+        warn(LD_REND,"http status 400 (\"%s\"). Dirserver didn't like our "
+             "rendezvous query?", reason);
         break;
       default:
-        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').",
+        warn(LD_REND,"http status %d (\"%s\") response unexpected (server "
+             "'%s:%d').",
              status_code, reason, conn->address, conn->port);
         break;
     }
@@ -1109,14 +1167,17 @@ connection_dir_client_reached_eof(connection_t *conn)
   if (conn->purpose == DIR_PURPOSE_UPLOAD_RENDDESC) {
     switch (status_code) {
       case 200:
-        info(LD_REND,"Uploading rendezvous descriptor: finished with status 200 (\"%s\")", reason);
+        info(LD_REND,"Uploading rendezvous descriptor: finished with status "
+             "200 (\"%s\")", reason);
         break;
       case 400:
-        warn(LD_REND,"http status 400 (\"%s\") response from dirserver '%s:%d'. Malformed rendezvous descriptor?",
+        warn(LD_REND,"http status 400 (\"%s\") response from dirserver "
+             "'%s:%d'. Malformed rendezvous descriptor?",
              reason, conn->address, conn->port);
         break;
       default:
-        warn(LD_REND,"http status %d (\"%s\") response unexpected (server '%s:%d').",
+        warn(LD_REND,"http status %d (\"%s\") response unexpected (server "
+             "'%s:%d').",
              status_code, reason, conn->address, conn->port);
         break;
     }
@@ -1132,16 +1193,7 @@ connection_dir_reached_eof(connection_t *conn)
   int retval;
   if (conn->state != DIR_CONN_STATE_CLIENT_READING) {
     info(LD_HTTP,"conn reached eof, not reading. Closing.");
-    /* This check is temporary; it's to let us know whether we should consider
-     * parsing partial serverdesc responses. */
-    if (conn->purpose == DIR_PURPOSE_FETCH_SERVERDESC &&
-        buf_datalen(conn->inbuf)>=(24*1024)) {
-      notice(LD_DIR,
-             "Directory connection closed early after downloading %d bytes "
-             "of descriptors. If this happens often, please file a bug report.",
-             (int)buf_datalen(conn->inbuf));
-    }
-    connection_close_immediate(conn); /* it was an error; give up on flushing */
+    connection_close_immediate(conn); /* error: give up on flushing */
     connection_mark_for_close(conn);
     return -1;
   }
@@ -1222,6 +1274,74 @@ already_fetching_directory(int purpose)
   return 0;
 }
 
+#undef INSTRUMENT_DOWNLOADS
+
+#ifdef INSTRUMENT_DOWNLOADS
+/** DOCDOC */
+static strmap_t *request_bytes_map = NULL;
+
+/** DOCDOC */
+static void
+note_request(const char *key, size_t bytes)
+{
+  uint64_t *n;
+  if (!request_bytes_map)
+    request_bytes_map = strmap_new();
+
+  n = strmap_get(request_bytes_map, key);
+  if (!n) {
+    n = tor_malloc_zero(sizeof(uint64_t));
+    strmap_set(request_bytes_map, key, n);
+  }
+  *n += bytes;
+}
+
+/** DOCDOC */
+char *
+directory_dump_request_log(void)
+{
+  smartlist_t *lines;
+  char tmp[256];
+  char *result;
+  strmap_iter_t *iter;
+
+  if (!request_bytes_map)
+    request_bytes_map = strmap_new();
+
+  lines = smartlist_create();
+
+  for (iter = strmap_iter_init(request_bytes_map);
+       !strmap_iter_done(iter);
+       iter = strmap_iter_next(request_bytes_map, iter)) {
+    const char *key;
+    void *val;
+    uint64_t *n;
+    strmap_iter_get(iter, &key, &val);
+    n = val;
+    tor_snprintf(tmp, sizeof(tmp), "%s  "U64_FORMAT"\n",
+                 key, U64_PRINTF_ARG(*n));
+    smartlist_add(lines, tor_strdup(tmp));
+  }
+  smartlist_sort_strings(lines);
+  result = smartlist_join_strings(lines, "", 0, NULL);
+  SMARTLIST_FOREACH(lines, char *, cp, tor_free(cp));
+  smartlist_free(lines);
+  return result;
+}
+#else
+static void
+note_request(const char *key, size_t bytes)
+{
+  return;
+}
+
+char *
+directory_dump_request_log(void)
+{
+  return tor_strdup("Not supported.");
+}
+#endif
+
 /** Helper function: called when a dirserver gets a complete HTTP GET
  * request.  Look for a request for a directory or for a rendezvous
  * service descriptor.  On finding one, write a response into
@@ -1247,14 +1367,17 @@ directory_handle_command_get(connection_t *conn, char *headers,
   }
   debug(LD_DIRSERV,"rewritten url as '%s'.", url);
 
-  if (!strcmp(url,"/tor/") || !strcmp(url,"/tor/dir.z")) { /* directory fetch */
+  if (!strcmp(url,"/tor/") || !strcmp(url,"/tor/dir.z")) { /* dir fetch */
     int deflated = !strcmp(url,"/tor/dir.z");
     dlen = dirserv_get_directory(&cp, deflated);
 
+    note_request(url, dlen);
+
     tor_free(url);
 
     if (dlen == 0) {
-      notice(LD_DIRSERV,"Client asked for the mirrored directory, but we don't have a good one yet. Sending 503 Dir not available.");
+      notice(LD_DIRSERV,"Client asked for the mirrored directory, but we "
+             "don't have a good one yet. Sending 503 Dir not available.");
       write_http_status_line(conn, 503, "Directory unavailable");
       /* try to get a new one now */
       if (!already_fetching_directory(DIR_PURPOSE_FETCH_DIR))
@@ -1280,8 +1403,9 @@ directory_handle_command_get(connection_t *conn, char *headers,
   if (!strcmp(url,"/tor/running-routers") ||
       !strcmp(url,"/tor/running-routers.z")) { /* running-routers fetch */
     int deflated = !strcmp(url,"/tor/running-routers.z");
-    tor_free(url);
     dlen = dirserv_get_runningrouters(&cp, deflated);
+    note_request(url, dlen);
+    tor_free(url);
     if (!dlen) { /* we failed to create/cache cp */
       write_http_status_line(conn, 503, "Directory unavailable");
       /* try to get a new one now */
@@ -1308,6 +1432,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
     size_t url_len = strlen(url);
     int deflated = !strcmp(url+url_len-2, ".z");
     smartlist_t *dir_objs = smartlist_create();
+    const char *request_type = NULL;
     const char *key = url + strlen("/tor/status/");
     if (deflated)
       url[url_len-2] = '\0';
@@ -1315,6 +1440,15 @@ directory_handle_command_get(connection_t *conn, char *headers,
       smartlist_free(dir_objs);
       return 0;
     }
+    if (!strcmpstart(key, "fp/"))
+      request_type = deflated?"/tor/status/fp.z":"/tor/status/fp";
+    else if (!strcmpstart(key, "authority"))
+      request_type = deflated?"/tor/status/authority.z":
+        "/tor/status/authority";
+    else if (!strcmpstart(key, "all"))
+      request_type = deflated?"/tor/status/all.z":"/tor/status/all";
+    else
+      request_type = "/tor/status/?";
     tor_free(url);
     if (!smartlist_len(dir_objs)) { /* we failed to create/cache cp */
       write_http_status_line(conn, 503, "Network status object unavailable");
@@ -1324,6 +1458,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
     dlen = 0;
     SMARTLIST_FOREACH(dir_objs, cached_dir_t *, d,
                       dlen += deflated?d->dir_z_len:d->dir_len);
+    note_request(request_type,dlen);
     format_rfc1123_time(date, time(NULL));
     tor_snprintf(tmp, sizeof(tmp),
                  "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
@@ -1350,9 +1485,22 @@ directory_handle_command_get(connection_t *conn, char *headers,
     int res;
     const char *msg;
     smartlist_t *descs = smartlist_create();
+    const char *request_type = NULL;
     if (deflated)
       url[url_len-2] = '\0';
     res = dirserv_get_routerdescs(descs, url, &msg);
+
+    if (!strcmpstart(url, "/tor/server/fp/"))
+      request_type = deflated?"/tor/server/fp.z":"/tor/server/fp";
+    else if (!strcmpstart(url, "/tor/server/authority"))
+      request_type = deflated?"/tor/server/authority.z":
+        "/tor/server/authority";
+    else if (!strcmpstart(url, "/tor/server/all"))
+      request_type = deflated?"/tor/server/all.z":"/tor/server/all";
+    else if (!strcmpstart(url, "/tor/server/d/"))
+      request_type = deflated?"/tor/server/d.z":"/tor/server/d";
+    else
+      request_type = "/tor/server/?";
     tor_free(url);
     if (res < 0)
       write_http_status_line(conn, 404, msg);
@@ -1389,6 +1537,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
                      "Content-Encoding: deflate\r\n\r\n",
                      date,
                      (int)compressed_len);
+        note_request(request_type, compressed_len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
         connection_write_to_buf(compressed, compressed_len, conn);
         tor_free(compressed);
@@ -1398,6 +1547,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
                      "Content-Type: text/plain\r\n\r\n",
                      date,
                      (int)len);
+        note_request(request_type, len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
         SMARTLIST_FOREACH(descs, signed_descriptor_t *, ri,
                           connection_write_to_buf(ri->signed_descriptor,
@@ -1423,7 +1573,8 @@ directory_handle_command_get(connection_t *conn, char *headers,
        * if we're gone to the site recently, and 404 if we haven't.
        *
        * Reject. */
-      write_http_status_line(conn, 400, "Nonauthoritative directory does not not store rendezvous descriptors");
+      write_http_status_line(conn, 400, "Nonauthoritative directory does not "
+                             "store rendezvous descriptors");
       tor_free(url);
       return 0;
     }
@@ -1435,6 +1586,7 @@ directory_handle_command_get(connection_t *conn, char *headers,
                      "Content-Type: application/octet-stream\r\n\r\n",
                      date,
                      (int)desc_len);
+        note_request("/tor/rendezvous?/", desc_len);
         connection_write_to_buf(tmp, strlen(tmp), conn);
         /* need to send descp separately, because it may include nuls */
         connection_write_to_buf(descp, desc_len, conn);
@@ -1450,6 +1602,22 @@ directory_handle_command_get(connection_t *conn, char *headers,
     return 0;
   }
 
+  if (!strcmpstart(url,"/tor/bytes.txt")) {
+    char *bytes = directory_dump_request_log();
+    size_t len = strlen(bytes);
+    format_rfc1123_time(date, time(NULL));
+    tor_snprintf(tmp, sizeof(tmp),
+                 "HTTP/1.0 200 OK\r\nDate: %s\r\nContent-Length: %d\r\n"
+                 "Content-Type: text/plain\r\n\r\n",
+                 date,
+                 (int)len);
+    connection_write_to_buf(tmp, strlen(tmp), conn);
+    connection_write_to_buf(bytes, len, conn);
+    tor_free(bytes);
+    tor_free(url);
+    return 0;
+  }
+
   /* we didn't recognize the url */
   write_http_status_line(conn, 404, "Not found");
   tor_free(url);
@@ -1476,7 +1644,8 @@ directory_handle_command_post(connection_t *conn, char *headers,
   if (!authdir_mode(get_options())) {
     /* we just provide cached directories; we don't want to
      * receive anything. */
-    write_http_status_line(conn, 400, "Nonauthoritative directory does not accept posted server descriptors");
+    write_http_status_line(conn, 400, "Nonauthoritative directory does not "
+                           "accept posted server descriptors");
     return 0;
   }
 
@@ -1497,7 +1666,7 @@ directory_handle_command_post(connection_t *conn, char *headers,
       case -2:
       case -1:
       case 1:
-        notice(LD_DIRSERV,"Rejected descriptor from %s.", origin);
+        notice(LD_DIRSERV,"Rejected router descriptor from %s.", origin);
         /* malformed descriptor, or something wrong */
         write_http_status_line(conn, 400, msg);
         break;
@@ -1511,10 +1680,20 @@ directory_handle_command_post(connection_t *conn, char *headers,
 
   if (!strcmpstart(url,"/tor/rendezvous/publish")) {
     /* rendezvous descriptor post */
-    if (rend_cache_store(body, body_len) < 0)
+    if (rend_cache_store(body, body_len) < 0) {
+//      char tmp[1024*2+1];
+      notice(LD_DIRSERV,"Rejected rend descriptor (length %d) from %s.",
+             (int)body_len, origin);
+#if 0
+      if (body_len <= 1024) {
+        base16_encode(tmp, sizeof(tmp), body, body_len);
+        notice(LD_DIRSERV,"Body was: %s", tmp);
+      }
+#endif
       write_http_status_line(conn, 400, "Invalid service descriptor rejected");
-    else
+    } else {
       write_http_status_line(conn, 200, "Service descriptor stored");
+    }
     goto done;
   }
 
@@ -1525,7 +1704,6 @@ directory_handle_command_post(connection_t *conn, char *headers,
   tor_free(url);
   tor_free(origin);
   return 0;
-
 }
 
 /** Called when a dirserver receives data on a directory connection;
@@ -1563,7 +1741,8 @@ directory_handle_command(connection_t *conn)
   else if (!strncasecmp(headers,"POST",4))
     r = directory_handle_command_post(conn, headers, body, body_len);
   else {
-    warn(LD_PROTOCOL,"Got headers '%s' with unknown command. Closing.", headers);
+    warn(LD_PROTOCOL,"Got headers '%s' with unknown command. Closing.",
+         headers);
     r = -1;
   }
 
