@@ -2,9 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: main.c,v 1.613 2006/01/11 04:04:42 nickm Exp $ */
+/* $Id: main.c,v 1.614 2006/01/11 22:34:07 arma Exp $ */
 const char main_c_id[] =
-  "$Id: main.c,v 1.613 2006/01/11 04:04:42 nickm Exp $";
+  "$Id: main.c,v 1.614 2006/01/11 22:34:07 arma Exp $";
 
 /**
  * \file main.c
@@ -625,13 +625,16 @@ run_connection_housekeeping(int i, time_t now)
            conn->address, conn->port, conn->s,
            (int)(now - conn->timestamp_created));
       conn->is_obsolete = 1;
-    } else if (connection_or_get_by_identity_digest(conn->identity_digest) !=
-               conn) {
-      info(LD_OR,
-           "Marking duplicate conn to %s:%d obsolete (fd %d, %d secs old).",
-           conn->address, conn->port, conn->s,
-           (int)(now - conn->timestamp_created));
-      conn->is_obsolete = 1;
+    } else {
+      connection_t *best =
+        connection_or_get_by_identity_digest(conn->identity_digest);
+      if (best && best != conn) {
+        info(LD_OR,
+             "Marking duplicate conn to %s:%d obsolete (fd %d, %d secs old).",
+             conn->address, conn->port, conn->s,
+             (int)(now - conn->timestamp_created));
+        conn->is_obsolete = 1;
+      }
     }
   }
 
