@@ -2,8 +2,9 @@
  * Copyright 2001-2004 Roger Dingledine.
  * Copyright 2004-2005 Roger Dingledine, Nick Mathewson. */
 /* See LICENSE for licensing information */
-/* $Id: relay.c,v 1.92 2005/12/10 09:36:26 arma Exp $ */
-const char relay_c_id[] = "$Id: relay.c,v 1.92 2005/12/10 09:36:26 arma Exp $";
+/* $Id: relay.c,v 1.98 2006/01/07 01:02:21 arma Exp $ */
+const char relay_c_id[] =
+  "$Id: relay.c,v 1.98 2006/01/07 01:02:21 arma Exp $";
 
 /**
  * \file relay.c
@@ -15,7 +16,8 @@ const char relay_c_id[] = "$Id: relay.c,v 1.92 2005/12/10 09:36:26 arma Exp $";
 
 static int relay_crypt(circuit_t *circ, cell_t *cell, int cell_direction,
                 crypt_path_t **layer_hint, char *recognized);
-static connection_t *relay_lookup_conn(circuit_t *circ, cell_t *cell, int cell_direction);
+static connection_t *relay_lookup_conn(circuit_t *circ, cell_t *cell,
+                                       int cell_direction);
 
 static int
 connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
@@ -35,11 +37,11 @@ circuit_consider_stop_edge_reading(circuit_t *circ, crypt_path_t *layer_hint);
 /** Stats: how many relay cells have originated at this hop, or have
  * been relayed onward (not recognized at this hop)?
  */
-unsigned long stats_n_relay_cells_relayed = 0;
+uint64_t stats_n_relay_cells_relayed = 0;
 /** Stats: how many relay cells have been delivered to streams at this
  * hop?
  */
-unsigned long stats_n_relay_cells_delivered = 0;
+uint64_t stats_n_relay_cells_delivered = 0;
 
 /** Update digest from the payload of cell. Assign integrity part to
  * cell.
@@ -113,9 +115,14 @@ relay_crypt_one_payload(crypto_cipher_env_t *cipher, char *in,
                         int encrypt_mode)
 {
   char out[CELL_PAYLOAD_SIZE]; /* 'in' must be this size too */
+  int r;
 
-  if (( encrypt_mode && crypto_cipher_encrypt(cipher, out, in, CELL_PAYLOAD_SIZE)) ||
-      (!encrypt_mode && crypto_cipher_decrypt(cipher, out, in, CELL_PAYLOAD_SIZE))) {
+  if (encrypt_mode)
+    r = crypto_cipher_encrypt(cipher, out, in, CELL_PAYLOAD_SIZE);
+  else
+    r = crypto_cipher_decrypt(cipher, out, in, CELL_PAYLOAD_SIZE);
+
+  if (r) {
     warn(LD_BUG,"Error during relay encryption");
     return -1;
   }
@@ -131,6 +138,8 @@ relay_crypt_one_payload(crypto_cipher_env_t *cipher, char *in,
  *    connection_edge.
  *  - Else connection_or_write_cell_to_buf to the conn on the other
  *    side of the circuit.
+ *
+ * Return -reason on failure.
  */
 int
 circuit_receive_relay_cell(cell_t *cell, circuit_t *circ, int cell_direction)
@@ -138,6 +147,7 @@ circuit_receive_relay_cell(cell_t *cell, circuit_t *circ, int cell_direction)
   connection_t *conn=NULL;
   crypt_path_t *layer_hint=NULL;
   char recognized=0;
+  int reason;
 
   tor_assert(cell);
   tor_assert(circ);
@@ -156,18 +166,21 @@ circuit_receive_relay_cell(cell_t *cell, circuit_t *circ, int cell_direction)
     if (cell_direction == CELL_DIRECTION_OUT) {
       ++stats_n_relay_cells_delivered;
       debug(LD_OR,"Sending away from origin.");
-      if (connection_edge_process_relay_cell(cell, circ, conn, NULL) < 0) {
+      if ((reason=connection_edge_process_relay_cell(cell, circ, conn, NULL))
+          < 0) {
         log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
-               "connection_edge_process_relay_cell (away from origin) failed.");
-        return -1;
+               "connection_edge_process_relay_cell (away from origin) "
+               "failed.");
+        return reason;
       }
     }
     if (cell_direction == CELL_DIRECTION_IN) {
       ++stats_n_relay_cells_delivered;
       debug(LD_OR,"Sending to origin.");
-      if (connection_edge_process_relay_cell(cell, circ, conn, layer_hint) < 0) {
+      if ((reason = connection_edge_process_relay_cell(cell, circ, conn,
+                                                       layer_hint)) < 0) {
         warn(LD_OR,"connection_edge_process_relay_cell (at origin) failed.");
-        return -1;
+        return reason;
       }
     }
     return 0;
@@ -185,18 +198,22 @@ circuit_receive_relay_cell(cell_t *cell, circuit_t *circ, int cell_direction)
   if (!conn) {
     if (circ->rend_splice && cell_direction == CELL_DIRECTION_OUT) {
       tor_assert(circ->purpose == CIRCUIT_PURPOSE_REND_ESTABLISHED);
-      tor_assert(circ->rend_splice->purpose == CIRCUIT_PURPOSE_REND_ESTABLISHED);
+      tor_assert(circ->rend_splice->purpose ==
+                 CIRCUIT_PURPOSE_REND_ESTABLISHED);
       cell->circ_id = circ->rend_splice->p_circ_id;
-      if (circuit_receive_relay_cell(cell, circ->rend_splice, CELL_DIRECTION_IN)<0) {
-        warn(LD_REND, "Error relaying cell across rendezvous; closing circuits");
-        circuit_mark_for_close(circ); /* XXXX Do this here, or just return -1? */
-        return -1;
+      if ((reason = circuit_receive_relay_cell(cell, circ->rend_splice,
+                                               CELL_DIRECTION_IN)) < 0) {
+        warn(LD_REND, "Error relaying cell across rendezvous; closing "
+             "circuits");
+        /* XXXX Do this here, or just return -1? */
+        circuit_mark_for_close(circ, -reason);
+        return reason;
       }
       return 0;
     }
     log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
            "Didn't recognize cell, but circ stops here! Closing circ.");
-    return -1;
+    return -END_CIRC_REASON_TORPROTOCOL;
   }
 
   debug(LD_OR,"Passing on unrecognized cell.");
@@ -238,7 +255,7 @@ relay_crypt(circuit_t *circ, cell_t *cell, int cell_direction,
 
   if (cell_direction == CELL_DIRECTION_IN) {
     if (CIRCUIT_IS_ORIGIN(circ)) { /* We're at the beginning of the circuit.
-                                     We'll want to do layered decrypts. */
+                                    * We'll want to do layered decrypts. */
       tor_assert(circ->cpath);
       thishop = circ->cpath;
       if (thishop->state != CPATH_STATE_OPEN) {
@@ -269,7 +286,8 @@ relay_crypt(circuit_t *circ, cell_t *cell, int cell_direction,
     } else { /* we're in the middle. Just one crypt. */
       if (relay_crypt_one_payload(circ->p_crypto, cell->payload, 1) < 0)
         return -1;
-//      log_fn(LOG_DEBUG,"Skipping recognized check, because we're not the OP.");
+//      log_fn(LOG_DEBUG,"Skipping recognized check, because we're not "
+//             "the OP.");
     }
   } else /* cell_direction == CELL_DIRECTION_OUT */ {
     /* we're in the middle. Just one crypt. */
@@ -371,7 +389,8 @@ relay_lookup_conn(circuit_t *circ, cell_t *cell, int cell_direction)
       return tmpconn;
     }
   }
-  for (tmpconn = circ->resolving_streams; tmpconn; tmpconn=tmpconn->next_stream) {
+  for (tmpconn = circ->resolving_streams; tmpconn;
+       tmpconn=tmpconn->next_stream) {
     if (rh.stream_id == tmpconn->stream_id && !tmpconn->marked_for_close) {
       debug(LD_EXIT,"found conn for stream %d.", rh.stream_id);
       return tmpconn;
@@ -428,7 +447,8 @@ connection_edge_send_command(connection_t *fromconn, circuit_t *circ,
   int cell_direction;
 
   if (fromconn && fromconn->marked_for_close) {
-    warn(LD_BUG,"Bug: called on conn that's already marked for close at %s:%d.",
+    warn(LD_BUG,
+         "Bug: called on conn that's already marked for close at %s:%d.",
          fromconn->marked_for_close_file, fromconn->marked_for_close);
     return 0;
   }
@@ -470,9 +490,10 @@ connection_edge_send_command(connection_t *fromconn, circuit_t *circ,
   debug(LD_OR,"delivering %d cell %s.", relay_command,
         cell_direction == CELL_DIRECTION_OUT ? "forward" : "backward");
 
-  if (circuit_package_relay_cell(&cell, circ, cell_direction, cpath_layer) < 0) {
+  if (circuit_package_relay_cell(&cell, circ, cell_direction, cpath_layer)
+      < 0) {
     warn(LD_BUG,"circuit_package_relay_cell failed. Closing.");
-    circuit_mark_for_close(circ);
+    circuit_mark_for_close(circ, END_CIRC_REASON_INTERNAL);
     return -1;
   }
   return 0;
@@ -487,7 +508,8 @@ connection_edge_end_reason_str(int reason)
 {
   switch (reason) {
     case -1:
-      warn(LD_PROTOCOL,"End cell arrived with length 0. Should be at least 1.");
+      warn(LD_PROTOCOL,
+           "End cell arrived with length 0. Should be at least 1.");
       return "MALFORMED";
     case END_STREAM_REASON_MISC:           return "misc error";
     case END_STREAM_REASON_RESOLVEFAILED:  return "resolve failed";
@@ -596,7 +618,8 @@ errno_to_end_reason(int e)
     E_CASE(EMFILE):
       return END_STREAM_REASON_RESOURCELIMIT;
     default:
-      info(LD_EXIT, "Didn't recognize errno %d (%s); telling the OP that we are ending a stream for 'misc' reason.",
+      info(LD_EXIT, "Didn't recognize errno %d (%s); telling the OP that "
+           "we are ending a stream for 'misc' reason.",
            e, tor_socket_strerror(e));
       return END_STREAM_REASON_MISC;
   }
@@ -631,12 +654,8 @@ connection_edge_process_end_not_open(
   routerinfo_t *exitrouter;
   int reason = *(cell->payload+RELAY_HEADER_SIZE);
 
-  if (rh->length > 0 && edge_reason_is_retriable(reason)) {
-    if (conn->type != CONN_TYPE_AP) {
-      warn(LD_PROTOCOL,"Got an end because of %s, but we're not an AP. Closing.",
-           connection_edge_end_reason_str(reason));
-      return -1;
-    }
+  if (rh->length > 0 && edge_reason_is_retriable(reason) &&
+      conn->type == CONN_TYPE_AP) {
     info(LD_APP,"Address '%s' refused due to '%s'. Considering retrying.",
          safe_str(conn->socks_request->address),
          connection_edge_end_reason_str(reason));
@@ -693,7 +712,8 @@ connection_edge_process_end_not_open(
             return 0;
           /* else, conn will get closed below */
         } else {
-          notice(LD_APP,"Have tried resolving address '%s' at %d different places. Giving up.",
+          notice(LD_APP,"Have tried resolving or connecting to address '%s' "
+                 "at %d different places. Giving up.",
                  safe_str(conn->socks_request->address), MAX_RESOLVE_FAILURES);
           /* clear the failures, so it will have a full try next time */
           client_dns_clear_failures(conn->socks_request->address);
@@ -739,11 +759,13 @@ connection_edge_process_relay_cell_not_open(
     connection_t *conn, crypt_path_t *layer_hint)
 {
   if (rh->command == RELAY_COMMAND_END)
-    return connection_edge_process_end_not_open(rh, cell, circ, conn, layer_hint);
+    return connection_edge_process_end_not_open(rh, cell, circ, conn,
+                                                layer_hint);
 
   if (conn->type == CONN_TYPE_AP && rh->command == RELAY_COMMAND_CONNECTED) {
     if (conn->state != AP_CONN_STATE_CONNECT_WAIT) {
-      warn(LD_APP,"Got 'connected' while not in state connect_wait. Dropping.");
+      warn(LD_APP,"Got 'connected' while not in state connect_wait. "
+           "Dropping.");
       return 0;
     }
 //    log_fn(LOG_INFO,"Connected! Notifying application.");
@@ -755,7 +777,8 @@ connection_edge_process_relay_cell_not_open(
       int ttl;
       if (!addr) {
         info(LD_APP,"...but it claims the IP address was 0.0.0.0. Closing.");
-        connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL, conn->cpath_layer);
+        connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL,
+                            conn->cpath_layer);
         connection_mark_unattached_ap(conn, END_STREAM_REASON_TORPROTOCOL);
         return 0;
       }
@@ -780,7 +803,8 @@ connection_edge_process_relay_cell_not_open(
     int ttl;
     int answer_len;
     if (conn->state != AP_CONN_STATE_RESOLVE_WAIT) {
-      warn(LD_APP,"Got a 'resolved' cell while not in state resolve_wait. Dropping.");
+      warn(LD_APP,"Got a 'resolved' cell while not in state resolve_wait. "
+           "Dropping.");
       return 0;
     }
     tor_assert(conn->socks_request->command == SOCKS_COMMAND_RESOLVE);
@@ -791,7 +815,8 @@ connection_edge_process_relay_cell_not_open(
       return 0;
     }
     if (rh->length >= answer_len+6)
-      ttl = (int)ntohl(get_uint32(cell->payload+RELAY_HEADER_SIZE+2+answer_len));
+      ttl = (int)ntohl(get_uint32(cell->payload+RELAY_HEADER_SIZE+
+                                  2+answer_len));
     else
       ttl = -1;
     connection_ap_handshake_socks_resolved(conn,
@@ -799,15 +824,18 @@ connection_edge_process_relay_cell_not_open(
                    cell->payload[RELAY_HEADER_SIZE+1], /*answer_len*/
                    cell->payload+RELAY_HEADER_SIZE+2, /*answer*/
                    ttl);
-    connection_mark_unattached_ap(conn, END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
+    connection_mark_unattached_ap(conn,
+                                  END_STREAM_REASON_ALREADY_SOCKS_REPLIED);
     return 0;
   }
 
   log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
          "Got an unexpected relay command %d, in state %d (%s). Dropping.",
-         rh->command, conn->state, conn_state_to_string(conn->type, conn->state));
+         rh->command, conn->state,
+         conn_state_to_string(conn->type, conn->state));
   return 0; /* for forward compatibility, don't kill the circuit */
-//  connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL, conn->cpath_layer);
+//  connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL,
+//                      conn->cpath_layer);
 //  connection_mark_for_close(conn);
 //  return -1;
 }
@@ -819,7 +847,7 @@ connection_edge_process_relay_cell_not_open(
  * If <b>layer_hint</b> is defined, then we're the origin of the
  * circuit, and it specifies the hop that packaged <b>cell</b>.
  *
- * Return -1 if you want to warn and tear down the circuit, else 0.
+ * Return -reason if you want to warn and tear down the circuit, else 0.
  */
 static int
 connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
@@ -829,6 +857,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
   static int num_seen=0;
   relay_header_t rh;
   unsigned domain = layer_hint?LD_APP:LD_EXIT;
+  int reason;
 
   tor_assert(cell);
   tor_assert(circ);
@@ -840,7 +869,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
 
   if (rh.length > RELAY_PAYLOAD_SIZE) {
     warn(LD_PROTOCOL, "Relay cell length field too long. Closing circuit.");
-    return -1;
+    return - END_CIRC_REASON_TORPROTOCOL;
   }
 
   /* either conn is NULL, in which case we've got a control cell, or else
@@ -875,7 +904,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
         connection_edge_end(conn, END_STREAM_REASON_TORPROTOCOL,
                             conn->cpath_layer);
         connection_mark_for_close(conn);
-        return -1;
+        return -END_CIRC_REASON_TORPROTOCOL;
       }
       debug(domain,"circ deliver_window now %d.", layer_hint ?
             layer_hint->deliver_window : circ->deliver_window);
@@ -890,7 +919,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
       if (--conn->deliver_window < 0) { /* is it below 0 after decrement? */
         log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
                "(relay data) conn deliver_window below 0. Killing.");
-        return -1; /* somebody's breaking protocol. kill the whole circuit. */
+        return -END_CIRC_REASON_TORPROTOCOL;
       }
 
       stats_n_data_bytes_received += rh.length;
@@ -945,14 +974,14 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
         return 0;
       }
       debug(domain,"Got an extended cell! Yay.");
-      if (circuit_finish_handshake(circ, CELL_CREATED,
-                                   cell->payload+RELAY_HEADER_SIZE) < 0) {
+      if ((reason = circuit_finish_handshake(circ, CELL_CREATED,
+                                       cell->payload+RELAY_HEADER_SIZE)) < 0) {
         warn(domain,"circuit_finish_handshake failed.");
-        return -1;
+        return reason;
       }
-      if (circuit_send_next_onion_skin(circ)<0) {
+      if ((reason=circuit_send_next_onion_skin(circ))<0) {
         info(domain,"circuit_send_next_onion_skin() failed.");
-        return -1;
+        return reason;
       }
       return 0;
     case RELAY_COMMAND_TRUNCATE:
@@ -961,12 +990,17 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
         return 0;
       }
       if (circ->n_conn) {
-        connection_send_destroy(circ->n_circ_id, circ->n_conn);
+        uint8_t reason = *(uint8_t*)(cell->payload + RELAY_HEADER_SIZE);
+        connection_or_send_destroy(circ->n_circ_id, circ->n_conn, reason);
         circuit_set_circid_orconn(circ, 0, NULL, N_CONN_CHANGED);
       }
       debug(LD_EXIT, "Processed 'truncate', replying.");
-      connection_edge_send_command(NULL, circ, RELAY_COMMAND_TRUNCATED,
-                                   NULL, 0, NULL);
+      {
+        char payload[1];
+        payload[0] = (char)END_CIRC_REASON_REQUESTED;
+        connection_edge_send_command(NULL, circ, RELAY_COMMAND_TRUNCATED,
+                                     payload, sizeof(payload), NULL);
+      }
       return 0;
     case RELAY_COMMAND_TRUNCATED:
       if (!layer_hint) {
@@ -979,7 +1013,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
       if (conn) {
         log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
                "'connected' unsupported while open. Closing circ.");
-        return -1;
+        return -END_CIRC_REASON_TORPROTOCOL;
       }
       info(domain,"'connected' received, no conn attached anymore. Ignoring.");
       return 0;
@@ -999,7 +1033,8 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
         return 0;
       }
       conn->package_window += STREAMWINDOW_INCREMENT;
-      debug(domain,"stream-level sendme, packagewindow now %d.", conn->package_window);
+      debug(domain,"stream-level sendme, packagewindow now %d.",
+            conn->package_window);
       connection_start_reading(conn);
       /* handle whatever might still be on the inbuf */
       if (connection_edge_package_raw_inbuf(conn, 1) < 0) {
@@ -1025,7 +1060,7 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
     case RELAY_COMMAND_RESOLVED:
       if (conn) {
         warn(domain,"'resolved' unsupported while open. Closing circ.");
-        return -1;
+        return -END_CIRC_REASON_TORPROTOCOL;
       }
       info(domain,"'resolved' received, no conn attached anymore. Ignoring.");
       return 0;
@@ -1043,7 +1078,8 @@ connection_edge_process_relay_cell(cell_t *cell, circuit_t *circ,
       return 0;
   }
   log_fn(LOG_PROTOCOL_WARN, LD_PROTOCOL,
-         "Received unknown relay command %d. Perhaps the other side is using a newer version of Tor? Dropping.",
+         "Received unknown relay command %d. Perhaps the other side is using "
+         "a newer version of Tor? Dropping.",
          rh.command);
   return 0; /* for forward compatibility, don't kill the circuit */
 }
@@ -1071,7 +1107,8 @@ connection_edge_package_raw_inbuf(connection_t *conn, int package_partial)
   tor_assert(conn);
   tor_assert(!connection_speaks_cells(conn));
   if (conn->marked_for_close) {
-    warn(LD_BUG,"Bug: called on conn that's already marked for close at %s:%d.",
+    warn(LD_BUG,
+         "Bug: called on conn that's already marked for close at %s:%d.",
          conn->marked_for_close_file, conn->marked_for_close);
     return 0;
   }
@@ -1088,7 +1125,8 @@ repeat_connection_edge_package_raw_inbuf:
     return 0;
 
   if (conn->package_window <= 0) {
-    info(domain,"called with package_window %d. Skipping.", conn->package_window);
+    info(domain,"called with package_window %d. Skipping.",
+         conn->package_window);
     connection_stop_reading(conn);
     return 0;
   }
@@ -1162,7 +1200,8 @@ connection_edge_consider_sending_sendme(connection_t *conn)
   }
 
   while (conn->deliver_window < STREAMWINDOW_START - STREAMWINDOW_INCREMENT) {
-    debug(conn->cpath_layer?LD_APP:LD_EXIT,"Outbuf %d, Queueing stream sendme.", (int)conn->outbuf_flushlen);
+    debug(conn->cpath_layer?LD_APP:LD_EXIT,
+          "Outbuf %d, Queueing stream sendme.", (int)conn->outbuf_flushlen);
     conn->deliver_window += STREAMWINDOW_INCREMENT;
     if (connection_edge_send_command(conn, circ, RELAY_COMMAND_SENDME,
                                      NULL, 0, conn->cpath_layer) < 0) {
@@ -1184,7 +1223,8 @@ circuit_resume_edge_reading(circuit_t *circ, crypt_path_t *layer_hint)
   debug(layer_hint?LD_APP:LD_EXIT,"resuming");
 
   /* have to check both n_streams and p_streams, to handle rendezvous */
-  if (circuit_resume_edge_reading_helper(circ->n_streams, circ, layer_hint) >= 0)
+  if (circuit_resume_edge_reading_helper(circ->n_streams, circ, layer_hint)
+      >= 0)
     circuit_resume_edge_reading_helper(circ->p_streams, circ, layer_hint);
 }
 
@@ -1201,7 +1241,8 @@ circuit_resume_edge_reading_helper(connection_t *conn,
     if (conn->marked_for_close)
       continue;
     if ((!layer_hint && conn->package_window > 0) ||
-        (layer_hint && conn->package_window > 0 && conn->cpath_layer == layer_hint)) {
+        (layer_hint && conn->package_window > 0 &&
+         conn->cpath_layer == layer_hint)) {
       connection_start_reading(conn);
       /* handle whatever might still be on the inbuf */
       if (connection_edge_package_raw_inbuf(conn, 1)<0) {
